@@ -28,9 +28,7 @@
 # include <readline/readline.h>
 #endif /*HAVE_LIBREADLINE*/
 
-#if GNUPG_MAJOR_VERSION != 1
 # include "gpg.h"
-#endif /*GNUPG_MAJOR_VERSION != 1*/
 #include "../common/util.h"
 #include "../common/i18n.h"
 #include "../common/ttyio.h"
@@ -39,13 +37,14 @@
 #include "main.h"
 #include "keyserver-internal.h"
 
-#if GNUPG_MAJOR_VERSION == 1
-# include "cardglue.h"
-#else /*GNUPG_MAJOR_VERSION!=1*/
-# include "call-agent.h"
-#endif /*GNUPG_MAJOR_VERSION!=1*/
+#include "call-agent.h"
 
 #define CONTROL_D ('D' - 'A' + 1)
+#define USER_PIN_DEFAULT "123456"
+#define ADMIN_PIN_DEFAULT "12345678"
+
+#define KDF_DATA_LENGTH_MIN  90
+#define KDF_DATA_LENGTH_MAX 110
 
 
 static void
@@ -62,8 +61,15 @@ write_sc_op_status (gpg_error_t err)
       write_status_text (STATUS_SC_OP_FAILURE, "1");
       break;
     case GPG_ERR_BAD_PIN:
+    case GPG_ERR_BAD_RESET_CODE:
       write_status_text (STATUS_SC_OP_FAILURE, "2");
       break;
+    case GPG_ERR_PIN_BLOCKED:
+      write_status_text (STATUS_SC_OP_FAILURE, "3");
+      break;
+    case GPG_ERR_NO_RESET_CODE:
+      write_status_text (STATUS_SC_OP_FAILURE, "4");
+      break;
     default:
       write_status (STATUS_SC_OP_FAILURE);
       break;
@@ -630,13 +636,7 @@ current_card_status (ctrl_t ctrl, estream_t fp,
                 const char *curve_for_print = "?";
 
                 if (info.key_attr[i].curve)
-                  {
-                    const char *oid;
-                    oid = openpgp_curve_to_oid (info.key_attr[i].curve,
-                                                NULL, NULL);
-                    if (oid)
-                      curve_for_print = openpgp_oid_to_curve (oid, 0);
-                  }
+                  curve_for_print = openpgp_oid_or_name_to_curve (info.key_attr[i].curve, 0);
                 tty_fprintf (fp, " %s", curve_for_print);
               }
           tty_fprintf (fp, "\n");
@@ -698,13 +698,21 @@ current_card_status (ctrl_t ctrl, estream_t fp,
       /* If the fingerprint is all 0xff, the key has no associated
          OpenPGP certificate.  */
       if ( thefpr && !fpr_is_ff (thefpr, thefprlen)
-           && !get_pubkey_byfprint (ctrl, pk, &keyblock, thefpr, thefprlen))
+           && !get_pubkey_byfpr (ctrl, pk, &keyblock, thefpr, thefprlen))
         {
           print_key_info (ctrl, fp, 0, pk, 0);
           print_card_key_info (fp, keyblock);
         }
       else
         tty_fprintf (fp, "[none]\n");
+
+      if (!info.manufacturer_name)
+        {
+          tty_fprintf (fp, "\n");
+          tty_fprintf (fp, _("Please try command \"%s\""
+                             " if the listing does not look correct\n"),
+                       "openpgp");
+        }
     }
 
   release_kbnode (keyblock);
@@ -713,6 +721,26 @@ current_card_status (ctrl_t ctrl, estream_t fp,
 }
 
 
+static void
+show_pin_hint (void)
+{
+  static int shown;
+
+  if (shown)
+    return;
+  shown = 1;
+
+  /* If no displayed name has been set, we assume that this is a fresh
+     card and print a hint about the default PINs.  */
+  tty_printf ("\n");
+  tty_printf (_("Please note that the factory settings of the PINs are\n"
+                "   PIN = '%s'     Admin PIN = '%s'\n"
+                "You should change them using the command --change-pin\n"),
+              USER_PIN_DEFAULT, ADMIN_PIN_DEFAULT);
+  tty_printf ("\n");
+}
+
+
 /* Print all available information for specific card with SERIALNO.
    Print all available information for current card when SERIALNO is NULL.
    Or print for all cards when SERIALNO is "all".  */
@@ -843,11 +871,12 @@ change_name (void)
     {
       tty_printf (_("Error: Combined name too long "
                     "(limit is %d characters).\n"), 39);
-      xfree (isoname);
       rc = gpg_error (GPG_ERR_TOO_LARGE);
       goto leave;
     }
 
+  show_pin_hint ();
+
   rc = agent_scd_setattr ("DISP-NAME", isoname, strlen (isoname));
   if (rc)
     log_error ("error setting Name: %s\n", gpg_strerror (rc));
@@ -909,8 +938,8 @@ fetch_url (ctrl_t ctrl)
         }
       else if (info.fpr1len)
 	{
-          rc = keyserver_import_fprint (ctrl, info.fpr1, info.fpr1len,
-                                        opt.keyserver, 0);
+          rc = keyserver_import_fpr (ctrl, info.fpr1, info.fpr1len,
+                                     opt.keyserver, 0);
 	}
     }
 
@@ -935,14 +964,6 @@ get_data_from_file (const char *fname, char **r_buffer)
   *r_buffer = NULL;
 
   fp = es_fopen (fname, "rb");
-#if GNUPG_MAJOR_VERSION == 1
-  if (fp && is_secured_file (fileno (fp)))
-    {
-      fclose (fp);
-      fp = NULL;
-      errno = EPERM;
-    }
-#endif
   if (!fp)
     {
       tty_printf (_("can't open '%s': %s\n"), fname, strerror (errno));
@@ -978,14 +999,6 @@ put_data_to_file (const char *fname, const void *buffer, size_t length)
   estream_t fp;
 
   fp = es_fopen (fname, "wb");
-#if GNUPG_MAJOR_VERSION == 1
-  if (fp && is_secured_file (fileno (fp)))
-    {
-      fclose (fp);
-      fp = NULL;
-      errno = EPERM;
-    }
-#endif
   if (!fp)
     {
       tty_printf (_("can't create '%s': %s\n"), fname, strerror (errno));
@@ -1290,6 +1303,7 @@ get_info_for_key_operation (struct agent_card_info_s *info)
   int rc;
 
   memset (info, 0, sizeof *info);
+  agent_scd_switchapp ("openpgp");
   rc = agent_scd_getattr ("SERIALNO", info);
   if (rc || !info->serialno || strncmp (info->serialno, "D27600012401", 12)
       || strlen (info->serialno) != 32 )
@@ -1418,6 +1432,7 @@ show_keysize_warning (void)
 }
 
 
+
 /* Ask for the size of a card key.  NBITS is the current size
    configured for the card.  Returns 0 to use the default size
    (i.e. NBITS) or the selected size.  */
@@ -1531,7 +1546,6 @@ ask_card_keyattr (int keyno, const struct key_attr *current)
   else
     {
       const char *curve;
-      const char *oid_str;
 
       if (current->algo == PUBKEY_ALGO_RSA)
         {
@@ -1552,8 +1566,7 @@ ask_card_keyattr (int keyno, const struct key_attr *current)
       if (curve)
         {
           key_attr->algo = algo;
-          oid_str = openpgp_curve_to_oid (curve, NULL, NULL);
-          key_attr->curve = openpgp_oid_to_curve (oid_str, 0);
+          key_attr->curve = openpgp_oid_or_name_to_curve (curve, 0);
         }
       else
         {
@@ -1605,6 +1618,8 @@ do_change_keyattr (int keyno, const struct key_attr *key_attr)
       return gpg_error (GPG_ERR_PUBKEY_ALGO);
     }
 
+  show_pin_hint ();
+
   err = agent_scd_setattr ("KEY-ATTR", args, strlen (args));
   if (err)
     log_error (_("error changing key attribute for key %d: %s\n"),
@@ -1637,6 +1652,7 @@ key_attr (void)
     {
       struct key_attr *key_attr;
 
+
       if ((key_attr = ask_card_keyattr (keyno, &info.key_attr[keyno])))
         {
           err = do_change_keyattr (keyno, key_attr);
@@ -1702,15 +1718,7 @@ generate_card_keys (ctrl_t ctrl)
   /* If no displayed name has been set, we assume that this is a fresh
      card and print a hint about the default PINs.  */
   if (!info.disp_name || !*info.disp_name)
-    {
-      tty_printf ("\n");
-      tty_printf (_("Please note that the factory settings of the PINs are\n"
-                    "   PIN = '%s'     Admin PIN = '%s'\n"
-                    "You should change them using the command --change-pin\n"),
-                  "123456", "12345678");
-      tty_printf ("\n");
-    }
-
+    show_pin_hint ();
 
   if (check_pin_for_key_operation (&info, &forced_chv1))
     goto leave;
@@ -1782,12 +1790,13 @@ card_generate_subkey (ctrl_t ctrl, kbnode_t pub_keyblock)
 }
 
 
-/* Store the key at NODE into the smartcard and modify NODE to
-   carry the serialno stuff instead of the actual secret key
-   parameters.  USE is the usage for that key; 0 means any
-   usage. */
+/* Store the key at NODE into the smartcard and modify NODE to carry
+   the serialno stuff instead of the actual secret key parameters.
+   USE is the usage for that key; 0 means any usage.  If
+   PROCESSED_KEYS is not NULL it is a pointer to an strlist which will
+   be filled with the keygrips of successfully stored keys.  */
 int
-card_store_subkey (KBNODE node, int use)
+card_store_subkey (KBNODE node, int use, strlist_t *processed_keys)
 {
   struct agent_card_info_s info;
   int okay = 0;
@@ -1796,8 +1805,9 @@ card_store_subkey (KBNODE node, int use)
   int  keyno;
   PKT_public_key *pk;
   gpg_error_t err;
-  char *hexgrip;
+  char *hexgrip = NULL;
   int rc;
+  char *ecdh_param_str = NULL;
   gnupg_isotime_t timebuf;
 
   log_assert (node->pkt->pkttype == PKT_PUBLIC_KEY
@@ -1871,15 +1881,31 @@ card_store_subkey (KBNODE node, int use)
     goto leave;
 
   epoch2isotime (timebuf, (time_t)pk->timestamp);
-  rc = agent_keytocard (hexgrip, keyno, rc, info.serialno, timebuf);
 
+  if (pk->pubkey_algo == PUBKEY_ALGO_ECDH)
+    {
+      ecdh_param_str = ecdh_param_str_from_pk (pk);
+      if (!ecdh_param_str)
+        {
+          err = gpg_error_from_syserror ();
+          goto leave;
+        }
+    }
+
+  rc = agent_keytocard (hexgrip, keyno, rc, info.serialno,
+                        timebuf, ecdh_param_str);
   if (rc)
     log_error (_("KEYTOCARD failed: %s\n"), gpg_strerror (rc));
   else
-    okay = 1;
-  xfree (hexgrip);
+    {
+      okay = 1;
+      if (processed_keys)
+        add_to_strlist (processed_keys, hexgrip);
+    }
 
  leave:
+  xfree (hexgrip);
+  xfree (ecdh_param_str);
   agent_release_card_info (&info);
   return okay;
 }
@@ -2057,11 +2083,6 @@ factory_reset (void)
 }
 
 
-#define USER_PIN_DEFAULT "123456"
-#define ADMIN_PIN_DEFAULT "12345678"
-#define KDF_DATA_LENGTH_MIN  90
-#define KDF_DATA_LENGTH_MAX 110
-
 /* Generate KDF data.  */
 static gpg_error_t
 gen_kdf_data (unsigned char *data, int single_salt)
@@ -2238,7 +2259,7 @@ enum cmdids
     cmdNAME, cmdURL, cmdFETCH, cmdLOGIN, cmdLANG, cmdSEX, cmdCAFPR,
     cmdFORCESIG, cmdGENERATE, cmdPASSWD, cmdPRIVATEDO, cmdWRITECERT,
     cmdREADCERT, cmdUNBLOCK, cmdFACTORYRESET, cmdKDFSETUP,
-    cmdKEYATTR, cmdUIF,
+    cmdKEYATTR, cmdUIF, cmdOPENPGP,
     cmdINVCMD
   };
 
@@ -2276,6 +2297,7 @@ static struct
       N_("setup KDF for PIN authentication (on/single/off)")},
     { "key-attr", cmdKEYATTR, 1, N_("change the key attribute")},
     { "uif", cmdUIF, 1, N_("change the User Interaction Flag")},
+    { "openpgp", cmdOPENPGP, 0, N_("switch to the OpenPGP app")},
     /* Note, that we do not announce these command yet. */
     { "privatedo", cmdPRIVATEDO, 0, NULL },
     { "readcert", cmdREADCERT, 0, NULL },
@@ -2575,6 +2597,11 @@ card_edit (ctrl_t ctrl, strlist_t commands)
             uif (arg_number, arg_rest);
           break;
 
+        case cmdOPENPGP:
+          agent_scd_switchapp ("openpgp");
+          redisplay = 1;
+          break;
+
         case cmdQUIT:
           goto leave;
 
